@@ -1,62 +1,67 @@
 import os
 import hashlib
 import json
-from django.contrib.staticfiles import finders
 from django.utils.safestring import mark_safe
-from .exceptions import SourceFileNotFound
-from .settings import STATIC_URL, STATIC_ROOT
-from .utils import render, bundle
+from django_webpack.models import WebpackBundle
+import exceptions
+import react
 
 
-class ReactComponent(object):
-    source = None
-    path_to_bundled_source = None
+class ReactComponent(WebpackBundle):
     props = None
     serialised_props = None
+    loaders = (
+        {'loader': 'jsx', 'test': '.jsx$'},
+    )
+    paths_to_loaders = (os.path.abspath(os.path.join(os.path.dirname(__file__), 'node_modules')),)
+    react_variable = 'window.React'
+    externals = {
+        # Rather than bundling React, we rely on a browser global
+        'react': react_variable,
+        'react/addons': react_variable
+    }
 
     def __init__(self, **kwargs):
         self.props = kwargs
 
-    def render(self):
-        return self.render_container(
-            content=self._render()
+    def render_to_string(self):
+        rendered = react.render_to_string(
+            path_to_source=self.get_path_to_source(),
+            serialised_props=self.get_serialised_props(),
         )
+        return self.render_container(content=rendered)
 
-    def render_static(self):
-        return self.render_container(
-            content=self._render(to_static_markup=True)
+    def render_to_static_markup(self):
+        rendered = react.render_to_static_markup(
+            path_to_source=self.get_path_to_source(),
+            serialised_props=self.get_serialised_props(),
         )
+        return self.render_container(content=rendered)
 
     def render_js(self):
-        rendered_js = '\n'.join((
+        rendered = '\n'.join((
             self.render_props(),
-            self.render_source(),
-            self.render_init()
+            self.render_bundle(),
+            self.render_init(),
         ))
-        return mark_safe(rendered_js)
+        return mark_safe(rendered)
 
     def render_container(self, content=None):
         if content is None:
             content = ''
-        rendered_container = '<div id="{container_id}" class="{container_class_name}">{content}</div>'.format(
+        rendered = '<div id="{container_id}" class="{container_class_name}">{content}</div>'.format(
             container_id=self.get_container_id(),
             container_class_name=self.get_container_class_name(),
             content=content,
         )
-        return mark_safe(rendered_container)
+        return mark_safe(rendered)
 
     def render_props(self):
-        rendered_props = '<script>{props_variable} = {serialised_props};</script>'.format(
+        rendered = '<script>{props_variable} = {serialised_props};</script>'.format(
             props_variable=self.get_props_variable(),
             serialised_props=self.get_serialised_props(),
         )
-        return mark_safe(rendered_props)
-
-    def render_source(self):
-        rendered_source = '<script src="{url_to_bundled_source}"></script>'.format(
-            url_to_bundled_source=self.get_url_to_bundled_source()
-        )
-        return mark_safe(rendered_source)
+        return mark_safe(rendered)
 
     def render_init(self):
         props_check = ''
@@ -67,9 +72,9 @@ class ReactComponent(object):
                 }}
             '''.format(
                 props_variable=self.get_props_variable(),
-                exported_variable=self.get_component_variable(),
+                exported_variable=self.get_library(),
             )
-        rendered_init = '''
+        rendered = '''
             <script>
                 if (!window.{exported_variable}) {{
                     throw new Error('Cannot find component `{exported_variable}`');
@@ -86,29 +91,12 @@ class ReactComponent(object):
             </script>
         '''.format(
             react_variable=self.get_react_variable(),
-            exported_variable=self.get_component_variable(),
+            exported_variable=self.get_library(),
             props_variable=self.get_props_variable(),
             container_id=self.get_container_id(),
             props_check=props_check,
         )
-        return mark_safe(rendered_init)
-
-    def _render(self, to_static_markup=None):
-        return render(
-            path_to_source=self.get_path_to_source(),
-            serialised_props=self.get_serialised_props(),
-            to_static_markup=to_static_markup,
-        )
-
-    def get_source(self):
-        return self.source
-
-    def get_path_to_source(self):
-        source = self.get_source()
-        path_to_source = finders.find(source)
-        if not os.path.exists(path_to_source):
-            raise SourceFileNotFound(path_to_source)
-        return path_to_source
+        return mark_safe(rendered)
 
     def get_props(self):
         return self.props
@@ -121,11 +109,13 @@ class ReactComponent(object):
     def get_component_name(self):
         return self.__class__.__name__
 
-    def get_component_variable(self):
+    def get_library(self):
+        if self.library:
+            return self.library
         return self.get_component_name()
 
     def get_react_variable(self):
-        return 'React'
+        return self.react_variable
 
     def get_component_id(self):
         return unicode(id(self))
@@ -142,7 +132,12 @@ class ReactComponent(object):
 
     def get_serialised_props(self):
         if not self.serialised_props:
-            self.serialised_props = json.dumps(self.get_props())
+            # Django will silently ignore json's exceptions, so we need to
+            # intercept them and raise our own class of exception
+            try:
+                self.serialised_props = json.dumps(self.get_props())
+            except (TypeError, AttributeError) as e:
+                raise exceptions.PropSerialisationError(e.__class__.__name__, *e.args)
         return self.serialised_props
 
     def get_serialised_props_hash(self):
@@ -155,46 +150,4 @@ class ReactComponent(object):
         return '__propsFor{component_name}_{serialised_props_hash}__'.format(
             component_name=self.get_component_name(),
             serialised_props_hash=self.get_serialised_props_hash(),
-        )
-
-    def generate_path_to_bundled_source(self, bundled_source):
-        md5 = hashlib.md5()
-        md5.update(bundled_source)
-        filename = '{component_name}-{hash}.js'.format(
-            component_name=self.get_component_name(),
-            hash=md5.hexdigest()
-        )
-        rel_path = os.path.join('django_react', 'bundles', filename)
-        return os.path.join(STATIC_ROOT, rel_path)
-
-    def write_bundled_source_file(self, bundled_source, path_to_file):
-        directory = os.path.dirname(path_to_file)
-        if not os.path.exists(directory):
-            os.makedirs(directory)
-        with open(path_to_file, 'w+') as bundled_source_file:
-            bundled_source_file.write(bundled_source)
-        return path_to_file
-
-    def generate_bundled_source_file(self):
-        bundled_source = bundle(
-            entry=self.get_path_to_source(),
-            library=self.get_component_variable(),
-        )
-        path_to_bundled_source = self.generate_path_to_bundled_source(bundled_source)
-        return self.write_bundled_source_file(bundled_source, path_to_bundled_source)
-
-    def get_path_to_bundled_source(self):
-        if not self.path_to_bundled_source:
-            self.path_to_bundled_source = self.generate_bundled_source_file()
-        return self.path_to_bundled_source
-
-    def get_rel_path_to_bundled_source(self):
-        path_to_bundled_source = self.get_path_to_bundled_source()
-        _, rel_path = path_to_bundled_source.split(STATIC_ROOT)
-        if rel_path.startswith('/') or rel_path.startswith('\\'):
-            rel_path = rel_path[1:]
-        return rel_path
-
-    def get_url_to_bundled_source(self):
-        rel_path_to_bundled_source = self.get_rel_path_to_bundled_source()
-        return os.path.join(STATIC_URL, rel_path_to_bundled_source)
\ No newline at end of file
+        )
\ No newline at end of file
